import 'package:flutter_waff/flutter_waff.dart';

import 'db_migration_create.dart';
import 'db_migration_upgrade.dart';

/// Database Migration
abstract class DbMigration {
  static const int CURRENT_VERSION = 4;

  /// Create
  static Future<void> onCreate(DbService db, int version) async {
    if (CURRENT_VERSION == 1) {
      await DbMigrationCreate.v1(db);
    } else if (CURRENT_VERSION == 2) {
      await DbMigrationCreate.v2(db);
    } else if (CURRENT_VERSION == 3) {
      await DbMigrationCreate.v3(db);
    } else if (CURRENT_VERSION == 4) {
      await DbMigrationCreate.v4(db);
    }
  }

  /// Upgrade
  static Future<void> onUpgrade(
      DbService db, int oldVersion, int newVersion) async {
    if (oldVersion == 1) {
      await DbMigrationUpgrade.v1toV2(db);
      await DbMigrationUpgrade.v2toV3(db);
      await DbMigrationUpgrade.v3toV4(db);
    } else if (oldVersion == 2) {
      await DbMigrationUpgrade.v2toV3(db);
      await DbMigrationUpgrade.v3toV4(db);
    } else if (oldVersion == 3) {
      await DbMigrationUpgrade.v3toV4(db);
    }
  }

  /// Downgrade
  static Future<void> onDowngrade(
      DbService db, int oldVersion, int newVersion) async {
    throw 'Unable to install the old version.';
  }
}
