import 'package:flutter_waff/flutter_waff.dart';
import 'package:get/get.dart';

import '../../../entities/repository_entity.dart';
import '../../../services/default_api/search_api.dart';

class RefreshController extends PullToRefreshController with HttpSmartCancel {
  final searchKey = 'flutter'.obs;
  final searchOrder = 'desc'.obs;
  final repositories = <RepositoryEntity>[].obs;

  @override
  bool get allowInitialRefresh => true;

  int _page = 1;

  @override
  void onClose() {
    super.onClose();
    cancelHttpRequest();
  }

  @override
  Future<int> startLoading() async {
    var reps = await SearchApi.repositories(HttpRunnerParams(
      query: {
        'q': searchKey.value,
        'page': _page + 1,
        'order': searchOrder.value,
      },
      cancelToken: getCancelToken(),
    ));
    if (reps == null) {
      return 0;
    }

    _page++;
    repositories.addAll(reps);
    isNoData.value = repositories.length <= 0;

    return reps.length;
  }

  @override
  Future<int> startRefresh() async {
    var reps = await SearchApi.repositories(HttpRunnerParams(
      query: {
        'q': searchKey.value,
        'page': 1,
        'order': searchOrder.value,
      },
      cancelToken: getCancelToken(),
    ));
    if (reps == null) {
      return 0;
    }

    _page = 1;
    repositories.clear();
    repositories.addAll(reps);
    isNoData.value = repositories.length <= 0;

    return reps.length;
  }
}
