import 'package:get/get.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';

abstract class PullToRefreshController extends GetxController {
  RefreshController get refreshController => _refreshController ??=
      RefreshController(initialRefresh: allowInitialRefresh);
  RefreshController? _refreshController;

  var isBusy = false.obs;

  var isNoData = true.obs;

  bool get allowInitialRefresh;

  Future<int> startRefresh();

  Future<int> startLoading();

  void onRefresh() async {
    isBusy.value = true;

    try {
      if (await startRefresh() <= 0) {
        isBusy.value = false;
        refreshController.loadNoData();
      }
    } catch (e) {
      isBusy.value = false;
      refreshController.refreshFailed();
      throw e;
    }

    isBusy.value = false;
    refreshController.refreshCompleted();
  }

  void onLoading() async {
    isBusy.value = true;

    try {
      if (await startLoading() <= 0) {
        isBusy.value = false;
        refreshController.loadNoData();
      }
    } catch (e) {
      isBusy.value = false;
      refreshController.loadFailed();
      throw e;
    }

    isBusy.value = false;
    refreshController.loadComplete();
  }
}
