import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';

import 'pull_to_refresh_controller.dart';

class PullToRefreshWidget extends StatelessWidget {
  final PullToRefreshController _controller;
  final Widget _child;
  final Widget _loadingWidget;
  final Widget _noDataWidget;
  final bool _enablePullUp;
  final bool _enablePullDown;

  PullToRefreshWidget(
      {required PullToRefreshController controller,
      required Widget child,
      Widget? loadingWidget,
      Widget? noDataWidget,
      bool enablePullUp = true,
      bool enablePullDown = true,
      Key? key})
      : this._controller = controller,
        this._child = child,
        this._loadingWidget = loadingWidget ?? Text('Loading...'),
        this._noDataWidget = noDataWidget ?? Text('No data!'),
        this._enablePullUp = enablePullUp,
        this._enablePullDown = enablePullDown,
        super(key: key);

  @override
  Widget build(BuildContext context) {
    return RefreshConfiguration(
        enableLoadingWhenNoData: false,
        child: Obx(() {
          return SmartRefresher(
            enablePullUp: _enablePullUp,
            enablePullDown: _enablePullDown,
            header: WaterDropMaterialHeader(),
            footer: ClassicFooter(
              loadStyle: LoadStyle.ShowWhenLoading,
            ),
            controller: _controller.refreshController,
            onRefresh: _controller.onRefresh,
            onLoading: _controller.onLoading,
            child: _controller.isNoData.value
                ? (_controller.isBusy.value ? _loadingWidget : _noDataWidget)
                : this._child,
          );
        }));
  }
}
