import 'package:flutter/material.dart';

import '../utils/utils.dart';
import '../waff_localizations.dart';

/// Show a alert-dialog
///
/// ```
/// await showAlertDialog('Your info ...', title: 'Info');
/// ```
Future<void> showAlertDialog(String content,
    {String? title, String? ok}) async {
  await showDialog(
    context: await getxContext,
    barrierDismissible: false,
    builder: (context) {
      return AlertDialog(
        title: !isEmptyOrNullString(title) ? Text(title!) : null,
        content: Text(content),
        actions: <Widget>[
          ElevatedButton(
            child:
                Text(isEmptyOrNullString(ok) ? WaffLocalizations.S('ok') : ok!),
            onPressed: () {
              Navigator.of(context).pop();
            },
          )
        ],
      );
    },
  );
}

/// Show a alert-dialog
///
/// ```
/// bool res = await confirmDialog('Your questions ...', title: 'Questions');
/// if (res) {
///   // ...
/// }
/// ```
Future<bool> showConfirmDialog(String content,
    {String? title, String? yes, String? no}) async {
  return (await showDialog<bool>(
      context: await getxContext,
      barrierDismissible: false,
      builder: (context) {
        return AlertDialog(
          title: !isEmptyOrNullString(title) ? Text(title!) : null,
          content: Text(content),
          actions: <Widget>[
            TextButton(
              child: Text(
                  isEmptyOrNullString(no) ? WaffLocalizations.S('no') : no!),
              onPressed: () {
                Navigator.of(context).pop(false);
              },
            ),
            ElevatedButton(
              child: Text(
                  isEmptyOrNullString(yes) ? WaffLocalizations.S('yes') : yes!),
              onPressed: () {
                Navigator.of(context).pop(true);
              },
            ),
          ],
        );
      }))!;
}
