import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import '../utils/utils.dart';
import '../waff_localizations.dart';

ScaffoldFeatureController<SnackBar, SnackBarClosedReason>? _snackBarController;

/// Show snack bar
///
/// ```
/// showSnackBar('message');
/// ```
void showSnackBar(String content,
    {String? title,
    int durationSeconds = 3,
    Color? backgroundColor,
    bool closePrevious = false}) async {
  Widget contentWidget = Text(content);
  if (title != null) {
    contentWidget = Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      mainAxisSize: MainAxisSize.min,
      children: [
        Text(
          title,
          style: TextStyle(fontWeight: FontWeight.bold),
        ),
        contentWidget,
      ],
    );
  }

  if (closePrevious) {
    _snackBarController?.close();
  }
  _snackBarController =
      ScaffoldMessenger.of(await getxContext).showSnackBar(SnackBar(
    content: contentWidget,
    duration: Duration(seconds: durationSeconds),
    shape: RoundedRectangleBorder(
      borderRadius: BorderRadius.circular(10.0),
    ),
    behavior: SnackBarBehavior.floating,
    backgroundColor: backgroundColor,
  ))
        ..closed.then((value) {
          if (value != SnackBarClosedReason.hide) {
            _snackBarController = null;
          }
        });
}

/// Show error snack bar
///
/// ```
/// showSnackBarError('error message');
/// ```
void showSnackBarError(String content,
    {String? title,
    int durationSeconds = 6,
    bool closePrevious = false}) async {
  showSnackBar(content,
      title: title ?? WaffLocalizations.S('an_error_occurred'),
      durationSeconds: durationSeconds,
      backgroundColor: Colors.red,
      closePrevious: closePrevious);
}
