import 'package:sqflite/sqflite.dart';

import 'db_service.dart';
import 'storage_service.dart';

abstract class DbMigrationHandler {
  static OnDatabaseCreateFn onCreate(DbService caller) {
    return (Database db, int version) async {
      var wrapper = DbService.wrapper(db);

      await wrapper.execute('''create table ${StorageService.TABLE_NAME}(
  "key" varchar(255) NOT NULL,
  "value" TEXT,
  PRIMARY KEY ("key")
)''');

      await caller.onCreate(wrapper, version);

      caller.log('Created database version: v$version');
    };
  }

  static OnDatabaseVersionChangeFn onUpgrade(DbService caller) {
    return (Database db, int oldVersion, int newVersion) async {
      var wrapper = DbService.wrapper(db);

      await caller.onUpgrade(wrapper, oldVersion, newVersion);

      caller.log(
          'Upgraded the database version from v$oldVersion to v$newVersion');
    };
  }

  static OnDatabaseVersionChangeFn onDowngrade(DbService caller) {
    return (Database db, int oldVersion, int newVersion) async {
      var wrapper = DbService.wrapper(db);

      await caller.onDowngrade(wrapper, oldVersion, newVersion);

      caller.log(
          'Downgrade the database version from v$oldVersion to v$newVersion');
    };
  }
}
