import 'dart:convert';

import 'db_service.dart';

/// Persistent storage service based on [DbService]
class StorageService {
  static const String TABLE_NAME = 'storage';

  final DbService _db;
  Map<String, dynamic> _data = {};

  StorageService(this._db);

  Future<void> init() async {
    var res = await _db.query(TABLE_NAME);
    res.forEach((element) {
      try {
        _data[element['key']?.toString() ?? ''] =
            jsonDecode(element['value']?.toString() ?? '');
      } catch (e) {
        return;
      }
    });
  }

  Future<void> setBool(String key, bool value) async {
    await _set(key, value);
  }

  Future<void> setInt(String key, int value) async {
    await _set(key, value);
  }

  Future<void> setDouble(String key, double value) async {
    await _set(key, value);
  }

  Future<void> setString(String key, String value) async {
    await _set(key, value);
  }

  Future<void> setJson(String key, dynamic value) async {
    await _set(key, value);
  }

  bool? getBool(String key) {
    return _data.containsKey(key) && _data[key] is bool ? _data[key] : null;
  }

  int? getInt(String key) {
    return _data.containsKey(key) && _data[key] is int ? _data[key] : null;
  }

  double? getDouble(String key) {
    return _data.containsKey(key) && _data[key] is double ? _data[key] : null;
  }

  String? getString(String key) {
    return _data.containsKey(key) && _data[key] is String ? _data[key] : null;
  }

  dynamic getJson(String key) {
    return _data.containsKey(key) ? _data[key] : null;
  }

  Future<void> _set(String key, dynamic value) async {
    _data[key] = value;

    if (await _db.exists(TABLE_NAME, where: 'key=?', whereArgs: [key])) {
      await _db.update(TABLE_NAME, {'value': jsonEncode(value)},
          where: 'key=?', whereArgs: [key]);
    } else {
      await _db.insert(TABLE_NAME, {'key': key, 'value': jsonEncode(value)});
    }
  }
}
