import 'package:flutter/widgets.dart';

import '../services/event_bus.dart';

/// AppLifecycle Listener
///
/// ```
/// // import packages
/// import 'package:flutter/widgets.dart';
/// import 'package:flutter_waff/flutter_waff.dart';
///
/// // Start listener
/// AppLifecycleListener.startup(WidgetsFlutterBinding.ensureInitialized());
/// // or
/// AppLifecycleListener.startup(WidgetsBinding.instance!);
///
/// // Listen AppLifecycleState
/// var appLifecycleEventStream = eventBus.on<AppLifecycleEvent>().listen((event) {
///   if (event.state == AppLifecycleState.resumed) {
///     ...
///   }
/// });
/// appLifecycleEventStream.cancel();
/// ```
///
class AppLifecycleListener {
  static AppLifecycleListener? _instance;

  WidgetsBinding _widgetsBindingInstance;
  _AppLifecycleListener _listener = _AppLifecycleListener();

  /// Start appLifecycle listen (single)
  factory AppLifecycleListener.startup(WidgetsBinding widgetsBindingInstance) {
    return _instance ??= AppLifecycleListener._(widgetsBindingInstance);
  }

  AppLifecycleListener._(this._widgetsBindingInstance) {
    _widgetsBindingInstance.addObserver(_listener);
  }

  /// Stop listener
  void stop() {
    _widgetsBindingInstance.removeObserver(_listener);
    _instance = null;
  }
}

class AppLifecycleEvent {
  final AppLifecycleState state;

  AppLifecycleEvent(this.state);
}

class _AppLifecycleListener with WidgetsBindingObserver {
  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    eventBus.fire(AppLifecycleEvent(state));
  }
}
