import 'dart:io';
import 'dart:ui';

import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:get/get.dart';

typedef LocaleDefine = Map<String, String>;
LocaleDefine _localeZhHans = {
  'ok': '确定',
  'cancel': '取消',
  'yes': '是',
  'no': '否',
  'an_error_occurred': '发生错误！',
  'system_setting': '系统设置',
  'authorize_in_system_setting': '请在 [系统设置] 中授权',
};
LocaleDefine _localeEn = {
  'ok': 'Ok',
  'cancel': 'Cancel',
  'yes': 'Yes',
  'no': 'No',
  'an_error_occurred': 'An Error Occurred!',
  'system_setting': 'System Settings',
  'authorize_in_system_setting': 'Please authorize in system settings',
};
LocaleDefine _localeJa = {
  'ok': '決定',
  'cancel': 'キャンセル',
  'yes': 'はい',
  'no': 'いいえ',
  'an_error_occurred': 'エラーが発生する！',
  'system_setting': 'システム設定',
  'authorize_in_system_setting': '[システム設定] で認証してください',
};

/// Waff language pack
///
/// ```
/// String langOk = WaffLocalizations.s('ok'); // Ok
/// String langHello = WaffLocalizations.s('hello', {'name': 'Jobs'}); // Hello Jobs!
/// ```
class WaffLocalizations {
  static Map<String, LocaleDefine> keys = {
    'en': _localeEn,
    'ja': _localeJa,
    'zh': _localeZhHans,
    'zh_CN': _localeZhHans,
  };

  static String _fallbackLocaleName = keys.keys.first;
  static String get fallbackLocaleName => _fallbackLocaleName;
  static set fallbackLocaleName(String value) {
    _fallbackLocaleName = keys.containsKey(value) ? value : keys.keys.first;
  }

  static const WaffLocalizationsDelegate delegate = WaffLocalizationsDelegate();

  final Locale locale;
  late final String currentName;

  WaffLocalizations(this.locale) {
    currentName =
        findLocaleName(keys.keys, locale.toString()) ?? _fallbackLocaleName;
  }

  static WaffLocalizations? of(BuildContext context) {
    return Localizations.of(context, WaffLocalizations);
  }

  static String S(String s, [Map<String, dynamic>? params]) {
    String? localeKey = _fallbackLocaleName;
    if (Get.context != null) {
      localeKey = of(Get.context!)?.currentName ?? _fallbackLocaleName;
    }
    s = keys[localeKey]![s] ?? '';

    if (params != null) {
      params.forEach((key, value) {
        s = s.replaceAll('@$key', value);
      });
    }

    return s;
  }
}

class WaffLocalizationsDelegate
    extends LocalizationsDelegate<WaffLocalizations> {
  const WaffLocalizationsDelegate();

  @override
  bool isSupported(Locale locale) {
    return findLocaleName(WaffLocalizations.keys.keys, locale.toString()) !=
        null;
  }

  @override
  Future<WaffLocalizations> load(Locale locale) {
    return SynchronousFuture<WaffLocalizations>(WaffLocalizations(locale));
  }

  @override
  bool shouldReload(LocalizationsDelegate<WaffLocalizations> old) {
    return false;
  }
}

/// If there are items related to [Platform.localeName]
/// in [supportedLocaleNames], return this item,
/// otherwise return [null].
String? findLocaleName(
    Iterable<String> supportedLocaleNames, String localeName) {
  if (supportedLocaleNames.contains(localeName)) {
    return localeName;
  }

  var lns = localeName.split('_');
  if (lns.length > 2) {
    var n = '${lns[0]}_${lns[2]}';
    if (supportedLocaleNames.contains(n)) {
      return n;
    }
  }

  if (supportedLocaleNames.contains(lns[0])) {
    return lns[0];
  }

  return null;
}
